﻿using System;
using Xamarin.Forms;

namespace GeometricFunc
{
    public class App : Application
    {
        private MasterDetailPage mainPage;

        public App()
        {
            InitializeStyles();
            mainPage = new MasterDetailPage();
            mainPage.IsGestureEnabled = false;

            SampleListPage sampleListPage = new SampleListPage();
            sampleListPage.SelectedSampleChanged += SampleListViewModel_SelectedSampleChanged;

            sampleListPage.SelectedSample = sampleListPage.Samples[0];
            mainPage.Master = sampleListPage;
            MainPage = mainPage;
        }

        private void SampleListViewModel_SelectedSampleChanged(object sender, EventArgs e)
        {
            ContentPage mainContentPage = null;
            SampleListPage sampleListViewModel = (SampleListPage)sender;
            BaseSampleView newSample = (BaseSampleView)Activator.CreateInstance(sampleListViewModel.SelectedSample.ClassType);

            if (mainPage.Detail == null)
            {
                mainContentPage = new ContentPage();
                mainPage.Detail = new NavigationPage(mainContentPage);
            }

            NavigationPage navigationPage = (NavigationPage)mainPage.Detail;
            mainContentPage = (ContentPage)navigationPage.CurrentPage;

            BaseSampleView oldSample = mainContentPage.Content as BaseSampleView;
            if (oldSample != null && !newSample.Equals(oldSample))
            {
                oldSample.Dispose();
            }

            mainContentPage.Content = newSample;
            mainContentPage.Title = newSample.Name;
            mainContentPage.ToolbarItems.Clear();
            mainContentPage.ToolbarItems.Add(newSample.SampleToolbarItem);
            mainPage.IsPresented = false;
        }

        private void InitializeStyles()
        {
            Application.Current.Resources = new ResourceDictionary();

            Style headerStyle = new Style(typeof(ContentView));
            headerStyle.Setters.Add(new Setter { Property = ContentView.PaddingProperty, Value = new Thickness(5, Device.OnPlatform(38, 5, 0), 5, 0) });
            Application.Current.Resources.Add("ListHeaderStyle", headerStyle);

            Style settingsTitleStyle = new Style(typeof(Label));
            settingsTitleStyle.Setters.Add(new Setter { Property = Label.TextColorProperty, Value = Color.Black });
            settingsTitleStyle.Setters.Add(new Setter { Property = Label.FontSizeProperty, Value = 21 });
            settingsTitleStyle.Setters.Add(new Setter { Property = Label.FontAttributesProperty, Value = FontAttributes.Bold });
            Application.Current.Resources.Add("SettingsTitleStyle", settingsTitleStyle);
        }
    }
}
