﻿using System;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Xml.Linq;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.FormsEdition;
using Xamarin.Forms;

namespace GeometricFunc
{
    public abstract class BaseSampleView : ContentView, IDisposable
    {
        protected static readonly GeoColor BrushColor = GeoColor.FromArgb(100, 0, 147, 221);

        private MapView mapView;
        private ToolbarItem sampleToolbarItem;
        private Collection<Feature> geometrySource;

        protected BaseSampleView()
        {
            mapView = new MapView();
            mapView.MapUnit = GeographyUnit.Meter;
            mapView.CurrentExtent = new RectangleShape(-30, 10, 10, -10);

            Content = new Grid { Children = { mapView } };

            var assembly = typeof(BaseSampleView).GetTypeInfo().Assembly;
            Stream stream = assembly.GetManifestResourceStream("GeometricFunc.AppData.GeometricFunctions.xml");
            XDocument xdoc = XDocument.Load(stream);
            XElement selectSampleElement = xdoc.Root.Elements().FirstOrDefault(e => e.Attribute("Name").Value == Name);
            XElement geometrySourceElement = selectSampleElement.Element("GeometrySource");
            foreach (var featureWktElement in geometrySourceElement.Elements())
            {
                if (!string.IsNullOrEmpty(featureWktElement.Value))
                {
                    GeometrySource.Add(new Feature(featureWktElement.Value));
                }
            }

            WorldMapKitOverlay worldOverlay = new WorldMapKitOverlay();
            worldOverlay.Projection = WorldMapKitProjection.SphericalMercator;
            mapView.Overlays.Insert(0, "WMK", worldOverlay);
            InitalizeMap();
        }

        public abstract string Name { get; }

        public MapView MapView
        {
            get { return mapView; }
        }

        public Collection<Feature> GeometrySource
        {
            get { return geometrySource ?? (geometrySource = new Collection<Feature>()); }
        }

        public virtual ToolbarItem SampleToolbarItem
        {
            get
            {
                if (sampleToolbarItem == null)
                {
                    sampleToolbarItem = new ToolbarItem { Text = "GO", Name = "GO" };
                    sampleToolbarItem.Clicked += delegate { ExecuteSample(); };
                }

                return sampleToolbarItem;
            }
        }

        protected virtual RectangleShape GetBoundingBox()
        {
            RectangleShape mapExtent = (RectangleShape)ExtentHelper.GetBoundingBoxOfItems(GeometrySource).CloneDeep();
            mapExtent.ScaleUp(100);
            return mapExtent;
        }

        protected abstract void ExecuteSample();

        protected abstract void InitalizeMap();

        ~BaseSampleView()
        {
            Dispose(false);
        }

        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        protected virtual void Dispose(bool disposing)
        {
            if (disposing)
            {
                mapView.Dispose();
            }
        }
    }
}
