﻿using System;
using System.Collections.ObjectModel;
using Xamarin.Forms;

namespace GeometricFunc
{
    public partial class SampleListPage : ContentPage
    {
        private SampleItem selectedSample;
        private ObservableCollection<SampleItem> samples;
        private Command openHelpUriCommand;

        public event EventHandler SelectedSampleChanged;

        public SampleListPage()
        {
            InitializeComponent();
            samples = new ObservableCollection<SampleItem>();

            samples.Add(new SampleItem("Union", "union", typeof(UnionSample)));
            samples.Add(new SampleItem("Difference", "difference", typeof(DifferenceSample)));
            samples.Add(new SampleItem("Buffer", "buffer", typeof(BufferSample)));
            samples.Add(new SampleItem("Scale", "scale", typeof(ScaleSample)));
            samples.Add(new SampleItem("Rotate", "rotate", typeof(RotateSample)));
            samples.Add(new SampleItem("CenterPoint", "centerPoint", typeof(CenterPointSample)));
            samples.Add(new SampleItem("GetArea", "getArea", typeof(GetAreaSample)));
            samples.Add(new SampleItem("Simplify", "simplify", typeof(SimplifySample)));
            samples.Add(new SampleItem("GetShortestLine To", "shortestLine", typeof(GetShortestLineSample)));
            samples.Add(new SampleItem("Get Length", "getLength", typeof(GetLengthSample)));
            samples.Add(new SampleItem("Line on Line", "lineOnLine", typeof(LineOnLineSample)));
            samples.Add(new SampleItem("Clip", "clip", typeof(ClipSample)));
            samples.Add(new SampleItem("Convex Hull", "convexHull", typeof(ConvexHullSample)));
            samples.Add(new SampleItem("Envelop Boundingbox", "bbox", typeof(EnvelopSample)));
            samples.Add(new SampleItem("Snapping", "snapGps", typeof(SnapGPSSample)));

            BindingContext = this;
        }

        public SampleItem SelectedSample
        {
            get
            {
                return selectedSample;
            }
            set
            {
                if (selectedSample != value)
                {
                    selectedSample = value;
                    OnPropertyChanged("SelectedSample");
                    OnSelectedSampleChanged();
                }
            }
        }

        public ObservableCollection<SampleItem> Samples
        {
            get { return samples; }
        }

        public Command OpenHelpUriCommand
        {
            get
            {
                openHelpUriCommand = openHelpUriCommand ?? new Command(() => Device.OpenUri(new Uri("http://www.thinkgeo.com")));
                return openHelpUriCommand;
            }
        }

        protected virtual void OnSelectedSampleChanged()
        {
            EventHandler handler = SelectedSampleChanged;
            if (handler != null)
            {
                handler(this, new EventArgs());
            }
        }
    }
}
